package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import ru.dbotthepony.mc.otm.*;
import ru.dbotthepony.mc.otm.capability.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.MatterHandlerCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.container.SimpleSerializableContainer;
import ru.dbotthepony.mc.otm.menu.MatterDecomposerMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;

public class BlockEntityMatterDecomposer extends BlockEntityMatteryPowered implements IItemHandler, IMatterGridCell {
	private static final TranslatableComponent MACHINE_NAME = new TranslatableComponent("block.overdrive_that_matters.matter_decomposer");
	private boolean valid = true;
	public final MatterHandlerCapability matter = new MatterHandlerCapability(this::setChanged, IMatterHandler.MatterDirection.EXTRACT, new BigDecimal("20"));
	private final LazyOptional<IMatterHandler> matter_resolver = LazyOptional.of(() -> matter);
	private final LazyOptional<IItemHandler> handler_resolver = LazyOptional.of(() -> this);

	// вход, выход
	public SimpleSerializableContainer item_container = new SimpleSerializableContainer(this::setChanged, 2);

	public BlockEntityMatterDecomposer(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.MATTER_DECOMPOSER, p_155229_, p_155230_);
		energy = new MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, new BigDecimal(400_000), new BigDecimal(2000), new BigDecimal(2000));
	}

	@Override
	public int getSlots() {
		return item_container.getContainerSize();
	}

	@Nonnull
	@Override
	public ItemStack getStackInSlot(int slot) {
		return item_container.getItem(slot);
	}

	@Nonnull
	@Override
	public ItemStack insertItem(int slot, @Nonnull ItemStack stack, boolean simulate) {
		if (slot != 0)
			return stack;

		if (!MatterRegistry.hasMatterValue(stack))
			return stack;

		ItemStack self_stack = item_container.getItem(slot);

		if (self_stack.isEmpty()) {
			if (!simulate) {
				item_container.setItem(slot, stack.copy());
			}

			return ItemStack.EMPTY;
		} else if (self_stack.isStackable() && self_stack.getMaxStackSize() > self_stack.getCount() && ItemStack.isSameItemSameTags(self_stack, stack)) {
			int new_count = Math.min(self_stack.getMaxStackSize(), self_stack.getCount() + stack.getCount());
			int diff = new_count - self_stack.getCount();

			if (diff != 0) {
				if (!simulate) {
					self_stack.grow(diff);
					item_container.setChanged();
				}

				ItemStack copy = stack.copy();
				copy.shrink(diff);
				return copy;
			}
		}

		return stack;
	}

	@Nonnull
	@Override
	public ItemStack extractItem(int slot, int amount, boolean simulate) {
		if (slot != 1 || amount == 0)
			return ItemStack.EMPTY.copy();

		if (amount < 0)
			throw new IllegalArgumentException("Can not extract negative amount of items");

		ItemStack self_stack = item_container.getItem(slot);

		if (self_stack.isEmpty())
			return ItemStack.EMPTY.copy();

		int minimal = Math.min(amount, self_stack.getCount());
		ItemStack copy = self_stack.copy();
		copy.setCount(minimal);

		if (!simulate) {
			self_stack.shrink(minimal);
			item_container.setChanged();
		}

		return copy;
	}

	@Override
	public int getSlotLimit(int slot) {
		// return Items.AIR.getMaxStackSize();
		return item_container.getMaxStackSize();
	}

	@Override
	public boolean isItemValid(int slot, @Nonnull ItemStack stack) {
		return slot == 0 && MatterRegistry.hasMatterValue(stack);
	}

	@Override
	protected Component getDefaultDisplayName() {
		return MACHINE_NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new MatterDecomposerMenu(containerID, inventory, this);
	}

	@Override
	public CompoundTag save(CompoundTag nbt) {
		nbt.put("work_inventory", item_container.serializeNBT());
		nbt.putDouble("work_progress", work_progress);
		nbt.put("work_item", work_item.serializeNBT());

		nbt.put("matter_capability", matter.serializeNBT());

		return super.save(nbt);
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);

		if (nbt.contains("matter_capability") && nbt.get("matter_capability") instanceof CompoundTag tag)
			matter.deserializeNBT(tag);

		if (nbt.contains("work_inventory") && nbt.get("work_inventory") instanceof CompoundTag tag)
			item_container = SimpleSerializableContainer.of(this::setChanged, tag, item_container.getContainerSize());

		work_progress = nbt.getDouble("work_progress");

		if (nbt.contains("work_item") && nbt.get("work_item") instanceof CompoundTag tag)
			work_item = ItemStack.of(tag);
	}

	@Override
	public void reviveCaps() {
		valid = true;
		super.reviveCaps();
	}

	@Override
	public void invalidateCaps() {
		valid = false;
		super.invalidateCaps();
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid) {
			if (cap == MatteryCapability.MATTER)
				return matter_resolver.cast();

			if (cap == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
				return handler_resolver.cast();
		}

		return super.getCapability(cap, side);
	}

	private static final BigDecimal ENERGY_CONSUMPTION_PER_TICK = new BigDecimal("240");

	private double work_progress = 0;
	private ItemStack work_item = ItemStack.EMPTY;

	public double getWorkProgress() {
		return work_progress;
	}

	public static <T extends BlockEntity> void tick(Level level, BlockPos blockPos, BlockState blockState, T t) {
		if (t instanceof BlockEntityMatterDecomposer tile) {
			tile.tickBatteryCharge();

			ItemStack stack = tile.item_container.getItem(0);

			if (!stack.isEmpty()) {
				BigDecimal matter_value = MatterRegistry.getMatterValue(stack);

				if (!matter_value.equals(BigDecimal.ZERO) && tile.matter.canReceiveAll(matter_value)) {
					if (!ItemStack.isSameItemSameTags(tile.work_item, stack)) {
						tile.work_progress = 0;
						tile.work_item = stack.copy();
					}

					tile.work_progress +=
						// work speed multiplier
						tile.workSpeedMultiplier(ENERGY_CONSUMPTION_PER_TICK) *
						// unit of full work for getting thing done (0.05 MtU is full unit of work, which is equal one real second)
						// the more MtUs are in item, the lesser this value should be
						(0.05 / matter_value.doubleValue()) *
						// the tick time
						0.05;

					if (tile.work_progress >= 1) {
						// work is done!
						tile.matter.receiveMatterInner(matter_value, false);
						stack.shrink(1);
						tile.item_container.setChanged();
						tile.work_progress = 0;
					}
				}
			} else if (tile.work_progress != 0) {
				tile.work_progress = 0;
				tile.work_item = ItemStack.EMPTY;
			}

			if (tile.matter.getStoredMatter().compareTo(BigDecimal.ZERO) > 0 && tile.grid != null) {
				BigDecimal diff = tile.matter.extractMatterInner(tile.matter.getStoredMatter(), true);
				BigDecimal diff2 = tile.grid.softPushMatter(diff, true);

				tile.matter.extractMatterInner(diff2, false);
				tile.grid.softPushMatter(diff2, false);
			}
		}
	}

	private MatterGrid grid;

	@Override
	public void setRemoved() {
		super.setRemoved();

		if (level != null && !level.isClientSide && grid != null)
			grid.untrack(this);
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (!p_155231_.isClientSide)
			MatterGrid.scheduleDiscoverNeighbours(this, getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Nullable
	@Override
	public IMatterHandler getMatterHandler() {
		return valid ? matter : null;
	}

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}
}
